import { Component, OnInit } from '@angular/core';
import { types } from './types';

export class Type {

  name: string;
  description: string;
  abstract: boolean;
  parent: Type;
  children: Type[];

  constructor(type: { name: string; description: string; superClasses?: string[]; properties?: any; abstract: boolean; }) {
    this.name = type.name;
    this.description = type.description;
    this.abstract = type.abstract;
    this.children = new Array();
  }

  addChild(type: Type) {
    type.parent = this;
    this.children.push(type);
  }

}


@Component({
  selector: 'app-resource-types-tree',
  templateUrl: './resource-types-tree.component.html',
  styleUrls: ['./resource-types-tree.component.css']
})
export class ResourceTypesTreeComponent implements OnInit {

  resources: Type[];

  map = new Map<string, Type>();

  unassociated = new Map<string, {
    name: string; description: string; superClasses?: string[];
    properties?: any[]; abstract: boolean;
  }>();

  constructor() { }

  analyseUnassociated(type: Type) {
    const associated: string[] = new Array();

    for (const name of this.unassociated.keys()) {
        const t = this.unassociated.get(name);
        const parent = t.superClasses[0];
        if (parent === type.name) {
          type.addChild(this.map.get(t.name));
          associated.push(t.name);
        }
    }

    for (const a of associated) {
      this.unassociated.delete(a);
    }

  }

  ngOnInit() {

    for (const t of types) {
      const type: Type = new Type(t);

      this.map.set(type.name, type);

      if (type.name !== 'Resource') {
        const parent = t.superClasses[0];
        if (this.map.has(parent)) {
          this.map.get(parent).addChild(type);
        } else {
          this.analyseUnassociated(type);
          this.unassociated.set(type.name, t);
        }
      }
    }

    this.resources = this.map.get('Resource').children;

  }

}



